// file: $USFS/class/image_analysis/v6.1/imaga_dct_06.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "image_analysis.h"
#include "image_analysis_constants.h"

// method: dct_zigzag_cc
//
// arguments: none
//
// return: a logical_1 value indicating status
//
// this method scans the DCT coefficients in a zigzag manner
//
logical_1 Image_analysis::dct_zigzag_cc() {
  
  // step size in each direction
  //
  int_4 step1 = -1;
  int_4 step2 = 1;

  // temporary array to store reordered values
  //
  float_4* temp_r = new float_4[dct_blk_size_d * dct_blk_size_d];
  float_4* temp_b = new float_4[dct_blk_size_d * dct_blk_size_d];
  float_4* temp_g = new float_4[dct_blk_size_d * dct_blk_size_d];

  memset(temp_r, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  memset(temp_b, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  memset(temp_g, 0, dct_blk_size_d * dct_blk_size_d * sizeof(float_4));
  
  // index variables
  //
  int_4 i = 0;
  int_4 j = 0;
  int_4 l = 0;
  
  // reorder the pel array in a zigzag fashion and store in temp array
  // need to do this only for the first coef_a positions
  //
  while (l < (dct_blk_size_d * dct_blk_size_d)) {
    
    // if the lower limit is exceeded reset in new direction
    //
    if ((i < 0) && (j < (int_4)dct_blk_size_d)) {
      i = 0;
      step1 *= -1;
      step2 *= -1;
    }
    
    else if ((i < (int_4)dct_blk_size_d) && (j < 0)) {
      j = 0;
      step1 *= -1;
      step2 *= -1;
    }
    
    // if the higher limit is exceeded reset in new direction
    //
    else if (i > (int_4)dct_blk_size_d - 1) {
      i = dct_blk_size_d - 1;
      j += 2;
      step1 *= -1;
      step2 *= -1;
    }
    
    else if (j > (int_4)dct_blk_size_d - 1) {
      j = dct_blk_size_d - 1;
      i += 2;
      step1 *= -1;
      step2 *= -1;
    }
    
    // otherwise rearrange data
    //
    else {
      temp_r[l] = dct_r_red_d[i * dct_blk_size_d + j];
      temp_g[l] = dct_r_grn_d[i * dct_blk_size_d + j];
      temp_b[l] = dct_r_blu_d[i * dct_blk_size_d + j];
      l++;
      i += step1;
      j += step2;
    }
  }

  // copy into the array and delete the memory
  //
  for (int_4 m=0; m < (dct_blk_size_d * dct_blk_size_d); m++) {
    dct_r_red_d[m] = temp_r[m];
    dct_r_grn_d[m] = temp_g[m];
    dct_r_blu_d[m] = temp_b[m];
  }

  delete [] temp_r;
  delete [] temp_g;
  delete [] temp_b;
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
