// file: $USFS/class/image_analysis/v1.0/imaga_fract_01.cc
//

// isip include files
//
#include "image_analysis.h"
#include "image_analysis_constants.h"

// method: fract_test_cc
//
// arguments:
//  float_8& sbe: (output) the image's sbe value
//  char_1* image_fname: (input) the image file name
//
// return: a logical_1 value indicating the status
//
// this method computes a fractal dimension and compares it to the
// reference model.
//
logical_1 Image_analysis::fract_test_cc(float_8& sbe_a,
				        char_1* image_fname_a) {
  
  // initialize the private data
  //
  if (fract_init_cc() == ISIP_FALSE) {
    error_handler_cc((char_1*)"fract_test_cc",
		     (char_1*)"error while initializing for the compression");
    return ISIP_FALSE;
  }
  
  // compute the fractal dimension
  //
  if (fract_compute_cc(image_fname_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"fract_test_cc",
		     (char_1*)"error while computing the fractal dimension");
    return ISIP_FALSE;
  }
  
  // provide some debugging information
  //
  if (debug_level_d > ISIP_DEBUG_BRIEF) {
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::fract_compute_cc",
	    "fract_npxls", fract_r_npxls_d);
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::fract_compute_cc",
	    "fract_r_nimages_d", fract_r_nimages_d);
  }
  
  // compute the sbe
  //
  if (fract_compare_cc(sbe_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"fract_test_cc",
  	     (char_1*)"error while computing the sbe");
    return ISIP_FALSE;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
