// file: $USFS/class/image_analysis/v6.1/imaga_hist_17.cc
//

// isip include files
//
#include "image_analysis.h"
#include "image_analysis_constants.h"

// method: hist_test_cc
//
// arguments:
//  float_8& distance: (output) the distance from the test feature vector to
//                     the feature vector in the model
//  uint_1* blk_r: (input) buffer for red color
//  uint_1* blk_g: (input) buffer for green color
//  uint_1* blk_b: (input) buffer for blue color
//  int_4 offset_w: (input) offset in width for the image block
//  int_4 offset_h: (input) offset in height for the image block
//  int_4 width: (input) width of the image block
//  int_4 height: (input) height of the image block
//
// return:
//  a logical_1 value indicating the status
//
// this method computes a histogram on an image block and compares it to
// the reference model
//
//
logical_1 Image_analysis::hist_test_cc(float_8& distance_a, uint_1* blk_r_a,
				       uint_1* blk_g_a, uint_1* blk_b_a,
				       int_4 offset_w_a, int_4 offset_h_a,
				       int_4 width_a, int_4 height_a) {
 
  // initialize
  //
  if (hist_init_cc() == ISIP_FALSE) {
    error_handler_cc((char_1*)"hist_test_cc",
		     (char_1*)"error while initializing the histogram");
    return ISIP_FALSE;
  }
  
  // compute a histogram
  //
  if (hist_compute_cc(blk_r_a, blk_g_a, blk_b_a, offset_w_a, offset_h_a,
		      width_a, height_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"hist_test_cc",
		     (char_1*)"error while computing the histogram");
    return ISIP_FALSE;
  }

  // normalize
  //
  if (hist_normalize_cc() == ISIP_FALSE) {
    error_handler_cc((char_1*)"hist_test_cc",
		     (char_1*)"error while normalizing the histogram");
    return ISIP_FALSE;
  }
  
  // provide some debugging information
  //
  if (debug_level_d > ISIP_DEBUG_BRIEF) {
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::hist_compute_cc",
	    "hist_t_npxls", hist_t_npxls_d);
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::hist_compute_cc",
	    "hist_t_nblks_d", hist_t_nblks_d);
  
    // display the histogram
    //
    fprintf(stdout, "red pdf:\n");
    for (int_4 i=0; i<hist_r_nbins_d; i++) {
      fprintf(stdout, "%ld %ld: %f\n", hist_r_redp_d, i, hist_t_rpdf_d[i]);
    }

    fprintf(stdout, "green pdf:\n");
    for (int_4 i=0; i<hist_r_nbins_d; i++) {
      fprintf(stdout, "%ld %ld: %f\n", hist_r_grnp_d, i, hist_t_gpdf_d[i]);
    }

    fprintf(stdout, "blue pdf:\n");
    for (int_4 i=0; i<hist_r_nbins_d; i++) {
      fprintf(stdout, "%ld %ld: %f\n", hist_r_blup_d, i, hist_t_bpdf_d[i]);
    }
  }
   
  // compute the distance
  //
  if (hist_compare_cc(distance_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"hist_test_cc",
		     (char_1*)"error while computing the distance");
    return ISIP_FALSE;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
