// file: $USFS/class/image_analysis/v6.1/imaga_rle_21.cc
//

// isip include files
//
#include "image_analysis.h"
#include "image_analysis_constants.h"

// method: rle_train_cc
//
// arguments:
//  uint_1* blk_r: (input) buffer for red color
//  uint_1* blk_g: (input) buffer for green color
//  uint_1* blk_b: (input) buffer for blue color
//  int_4 offset_w: (input) offset in width for the image block
//  int_4 offset_h: (input) offset in height for the image block
//  int_4 width: (input) width of the image block
//  int_4 height: (input) height of the image block
//
// return:
//  a logical_1 value indicating status
//
// this method computes the compression ratio on a prescribed region in the
// training mode
//
logical_1 Image_analysis::rle_train_cc(uint_1* blk_r_a, uint_1* blk_g_a,
				       uint_1* blk_b_a, int_4 offset_w_a,
				       int_4 offset_h_a, int_4 width_a,
				       int_4 height_a) {
  
  // compute the compression ratio
  //
  if (rle_compute_cc(blk_r_a, blk_g_a, blk_b_a, offset_w_a, offset_h_a,
		     width_a, height_a) == ISIP_FALSE) {
    error_handler_cc((char_1*)"rle_train_cc",
		     (char_1*)"error while computing the compression ratio");
    return ISIP_FALSE;
  }

  // provide some debugging information
  //
  if (debug_level_d > ISIP_DEBUG_BRIEF) {
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::rle_compute_cc",
	    "rle_npxls", rle_r_npxls_d);
    fprintf(stdout, "<%s> %s = %ld\n", "Image_analysis::rle_compute_cc",
	    "rle_r_nimages_d", rle_r_nimages_d);
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
